# smartClientCore - Setup

## Table of contents
- [Global Config](#global-config)
- [Sequential Config](#sequential-config)
  
## Global Config
- [debug](#debug)
- [blockInsecureURL](#blockinsecureurl)
- [maxWrapperDepth](#maxwrapperdepth)
- [adReinsertion](#adreinsertion)
  - [HOMAD](#adreinsertion+homad)
- [adVerification](#adverification)
  - [OMID](#adverification+omid)
  - [nonOMID](#adverification+nonomid)
- [skipOffset](#skipoffset)
- [timeouts](#timeouts)
  - [adRequest](#timeouts+adrequest)
  - [adRequestChain](#timeouts+adrequestchain)
  - [adSlotRequest](#timeouts+adslotrequest)
  - [videoRequest](#timeouts+videorequest)
  - [vpaidRequest](#timeouts+vpaidrequest)
  - [vpaidReadiness](#timeouts+vpaidreadiness)

<a name="debug"></a>

### debug : <code>Boolean</code>
Enables log output to the console.

<a name="blockinsecureurl"></a>

### blockInsecureURL : <code>Boolean</code>
By default insecure URL requests (tags, tracker, media files) will not be executed within secure environment. Default is `true`.

<a name="maxwrapperdepth"></a>

### maxWrapperDepth : <code>number</code>
When serving an ad involves a chain of wrappers, an infinite loop is possible where a chain of Wrappers never results in a final inline VAST response. 
In general, wrappers should be limited to ten before resulting in an inline response. If the add-on detects more than ten wrappers,
the add-on rejects any subsequent responses in the chain and returns VAST error 302.
Default limit is `10` wrappers.

<a name="adreinsertion"></a>

### adReinsertion : <code>Object</code>

<a name="adreinsertion+homad"></a>

#### homad : <code>Object</code>
The add-on has been extended with the **HOMAD HTML5 solution** to help bypassing ad-blockers.
All ad-related resources (tags, creative files, trackers) will be redirected to HOMAD once they get blocked for any reason.

**enabled : <code>Boolean</code>** - Enables HOMAD ad-reinsertion. Disabled by default.

**setup : <code>Object</code>** - HOMAD configuration

| Param | Type | Description |
| --- | --- | --- |
| globalConfig | <code>string</code> | URI to HOMAD's global configuration file. |
| clientConfig | <code>string</code> | URI to the client-specific configuration file. |
| enableSessionCoverage | <code>Boolean</code> | Decide whether ad-blockers should be detected again when requesting a new ad-break (default behavior) or not.<br><br>Disabled by default. |
| activationCallback([reason <code>number</code>](#adreinsertion+activation-codes)) | ⇒ <code>Promise</code> | Is called whenever the add-on detects a scenario that blocks or would block an ad-related request.<br>Based on the specified [reason](#adreinsertion+activation-codes) the player has to decide whether the current ad-slot must be requested again through HOMAD or not.<br><br>Resolves if the publisher decides to re-call the current ad-break through HOMAD. |

> _If the `activationCallback` is omitted or does not respond within 250 milliseconds, 
> HOMAD will be automatically activated (if `enabled` before)._


```javascript
let globalAdConfig = smartclientcore.SCHEMA.create(smartclientcore.SCHEMA.GLOBAL_CONFIG);
globalAdConfig.adReinsertion = {
	homad: {
		enabled: true,
		setup: {
			globalConfig: 'https://s3.amazonaws.com/homad-global-configs.schneevonmorgen.com/global_config.json',
			clientConfig: 'https://test.s3.amazonaws.com/homad/homadConfigTestHttps.json',
			enableSessionCoverage: true,
			activationCallback: (reason) => {
				return new Promise((resolve, reject) => {
					
					// The reason for activating HOMAD sounds reasonable?
					resolve();
					
					// otherwise
					//reject();
				});
			}
		}
	}
};
```

<a name="adreinsertion+activation-codes"></a>

#### Activation Reason - Codes Table

| Code | ENUM | Description |
| --- | --- | --- |
| 1000 | `AdReinsertionModel.REASON.XHR_TIMEOUT` | Timeout of ad-request URIs |
| 1001 | `AdReinsertionModel.REASON.XHR_UNEXPECTED_STATUS` | XHR request status is not 200 (ok)  |
| 1010 | `AdReinsertionModel.REASON.MEDIAFILE_TIMEOUT` | Timeout of media file request URIs |
| 1013 | `AdReinsertionModel.REASON.MEDIAFILE_LOAD_ERROR` | General load error of media file URIs |


<a name="adverification"></a>

### adVerification : <code>Object</code>

<a name="adverification+omid"></a>

#### OMID : <code>Object</code>
Ad-verification vendors that rely on the Open Measurement Interface Definition API (**OMID**) are enabled by default.
The following properties may be adjusted to enable selected verification vendors only and to limit the measurement session execution time.

More information on OMID and OM SDK available at: [https://www.iab.com/omsdk](https://www.iab.com/omsdk)

**vendorWhitelist : <code>[ &#x27;Array&#x27; ].&lt;string&gt;</code>** - List of accepted AdVerification vendors (whitelist).
Each specified String has to represent an unique identifier that can be found within the verification code resource URI, 
most likely the domain name.

If omitted, the add-on tries to load verification code from each vendor.

**sessionCleanupDelay : <code>number</code>** - Time value in seconds that specifies a delay for closing the current measurement session
after the ad has finished playback. 
Default is `3` seconds.

> _The verification code should have sufficient time to ensure any measurement 
and reporting is completed before cleaning up, in order to minimize discrepancies._ 

**timeoutScriptLoad : <code>number</code>** - Representing the number of seconds a OMID vendor can take to load and initialize its code before being terminated.
Defaulting to `0.5` seconds if omitted.

**playerHandles : <code>Boolean</code>** - Whether the player takes responsibility for Open Measurement, e.g. native implementations of OMID. 
Defaulting to `false` if omitted.


<a name="adverification+nonomid"></a>

#### nonOMID : <code>Object</code>
Ad-verification vendors that base or rely on proprietary measurement and reporting logic.

**meetrics : <code>Boolean</code>** - Disabled by default.
Enables Meetrics viewability measurement for video advertisment with any VAST and VPAID creative.


<a name="skipoffset"></a>

### skipOffset : <code>number</code>
Time value in seconds that identifies when skip controls are made available to the end user by default.
Advertisers may define a `skipoffset` value in its VAST tag that has higher priority than the publisher setting.


<a name="timeouts"></a>

### timeouts : <code>Object</code>
```javascript
let globalAdConfig = smartclientcore.SCHEMA.create(smartclientcore.SCHEMA.GLOBAL_CONFIG);
globalAdConfig.timeouts = {
	adRequest: 1,
	adRequestChain: 2,
	adSlotRequest: 4,
	videoRequest: 3,
	vpaidRequest: 1,
	vpaidReadiness: 3,
};
```

<a name="timeouts+adrequest"></a>

#### adRequest : <code>number</code>
The number of seconds a single ad request can take to load before being terminated.  
Default is `1` seconds.
 
<a name="timeouts+adrequestchain"></a>

#### adRequestChain : <code>number</code>
The total number of seconds a single ad request chain (VAST Wrappers and final VAST Inline tag) can take to load before being terminated.  
Default is `2` seconds.

<a name="timeouts+adslotrequest"></a>

#### adSlotRequest : <code>number</code>
The total number of seconds an ad-slot request (from calling `initAdSlot` until the Promise resolves or rejects) can take before being terminated.  
Default is `4` seconds.

<a name="timeouts+videorequest"></a>

#### videoRequest : <code>number</code>
Representing the number of seconds a video creative can take to load before being terminated.  
Default is `3` seconds.

<a name="timeouts+vpaidrequest"></a>

#### vpaidRequest : <code>number</code>
Representing the number of seconds a VPAID creative (JavaScript resource) can take to load before being terminated.  
Default is `1` seconds.

<a name="timeouts+vpaidreadiness"></a>

#### vpaidReadiness : <code>number</code>
Representing the number of seconds a VPAID creative can take to initialize and respond 
(from calling `VPAID.initAd()` until the creative triggers the `AdLoaded` event) before being terminated.  
Default is `3` seconds.


## Sequential Config

<a name="AdBreak"></a>

### adBreak : <code>AdBreak</code>
Setup for an ad-break that defines the sequence of commercial ads, sponsored ads and separation clips.

Commercial and sponsored ads (type NORMAL and SPONSORED) are defined by the ad-server VAST manifest.

Separation clips (type OPENER, CLOSER, BUMPER) are optional. They can be defined by the publisher. 
Most likely they depend on the upcoming TV show or editorial video.

**tag : <code>[string, [ &#x27;Array&#x27; ].&lt;string&gt;]</code>** - Mandatory <sup>*</sup>. Defines the ad-request URL(s) that return a VAST manifest. Allowed values are:
 - a single URL, ex. ad fill rate is ensured by the ad-server, a multiple ads ad-break is defined by the ad-server 
 - an Array of URLs, ex. use Waterfalling with unknown ad fill rates, manual multiple ads ad-break setup

**response : <code>string.<XML></code>** - Mandatory <sup>*</sup>. Defines a VAST manifest directly as a String.

> _<sup>*</sup> One of the **tag** or **response** property is required but both are not supported. If both have been specified, **tag** will get preferred._

```javascript
adPlugin.initAdSlot(
	{
		tag: 'https://adserver.com/tag-pre?type=vast3xml'
	}
);
// or...
adPlugin.initAdSlot(
	{
		response: `<VAST version="4.0">
                   	<Ad id="example-inline-tag">
                   		<InLine>
                   			<!-- example code -->
                   		</InLine>
                   	</Ad>
                   </VAST>`
	}
);
```

**limit : <code>number</code>** - Optional _and applicable with Waterfalling only_. Defines the maximum number of impressions to be delivered from a chain of ad-requests.

> _All SPONSORED ads are also counted as valid impression. The limit value should be increased accordingly_

```javascript
adPlugin.initAdSlot({
    tag: [
        'https://adserver-1.com/tag-pre?type=vast3xml',
        'https://adserver-2.com/tag-pre?type=vast3xml',
        'https://fallback-adserver.com/tag-pre?type=vast3xml'
    ],
    limit: 2
}).then(adLoaded => {
    console.log('### Loading AdManifest success.');

    if (autoplay) {
        adPlugin.startAdSlot();
    }

}, failedLoading => {
    console.warn('### Loading AdManifest failed', failedLoading);
});
```

**opener : <code>Object</code>** - _Optional_. Applies to NORMAL and SPONSORED ads.
Intro video for the upcoming ad break, played before all other ads.

- **progressive : <code>string</code>** - _Mandatory_. An Array of objects to video files, used for progressive download.
 Multiple progressive files can be used, e.g. .mp4 and .webm files to support as many browsers as possible.
 
- **forcePlay : <code>Boolean</code>** - _Optional_. Defines whether the separation clip must be delivered with ads or always. Defaulting to `false`.

Other source properties are currently not supported nor required by the add-on. 
Future versions of the add-on may include streaming resources.

**closer : <code>Object</code>** - _Optional_. Applies to NORMAL and SPONSORED ads.
Outro video, played after all other ads.

- **progressive : <code>string</code>** - _Mandatory_. An Array of objects to video files, used for progressive download.
 Multiple progressive files can be used, e.g. .mp4 and .webm files to support as many browsers as possible.

- **forcePlay : <code>Boolean</code>** - _Optional_. Defines whether the separation clip must be delivered with ads or always. Defaulting to `false`.

Other source properties are neither supported nor required by the add-on.

**bumper : <code>Object</code>** - _Optional_. Applies to NORMAL and SPONSORED ads.
Special intro video, played once when switching from commercial to sponsored ads or vice versa.
Not served if the ad-server response contains no ads of variant NORMAL or SPONSORED.

- **progressive : <code>string</code>** - _Mandatory_. An Array of objects to video files, used for progressive download.
 Multiple progressive files can be used, e.g. .mp4 and .webm files to support as many browsers as possible.

- **forcePlay : <code>Boolean</code>** - _Optional_. Defines whether the separation clip must be delivered with ads or always. Defaulting to `false`.

Other source properties are neither supported nor required by the add-on.

```javascript
adPlugin.initAdSlot(
	{
		tag: 'https://adserver.com/tag-pre?type=vast3xml',
		opener: {
			progressive: 'https://yourpage.com/assets/opener-pre.mp4'
		},
		closer: {
			progressive: 'https://yourpage.com/assets/closer-pre.mp4'
		},
		bumper: {
			progressive: 'https://yourpage.com/assets/bumper-pre.mp4'
		}
	}
);
```

Example setup to play a static "Station Bumper"
```javascript
adPlugin.initAdSlot(
	{
		tag: '',
		opener: {
			progressive: 'https://yourpage.com/assets/station-bumper.mp4',
			forcePlay: true
		}
	}
);
```

### environmentVars : <code>EnvironmentVars</code>

Used for passing implementation-specific runtime variables.

**desiredBitrate : <code>number</code>** - _Optional_. Indicates the desired bitrate in kilobits per second (kbps). 
The player uses this information to select media files with an appropriate bitrate. Defaulting to `null`.  
**_Note_**_: If set, this will disable the plugin-internal bitrate detection._

**desiredMimeTypes : <code>[ &#x27;Array&#x27; ].&lt;string&gt;</code>** - _Optional_. Declaration of accepted media types (whitelist). 
Defaulting to types that the browser eventually is able to play. Types must be compliant with the [IANA Media Type standard](https://www.iana.org/assignments/media-types/media-types.xhtml)  
**_Note_**_: If set, this will override the plugin-internal media types. For a list of supported media types see [tbd - Glossar Media Types](#see-elsewhere)._

**desiredVPAIDVendors : <code>[ &#x27;Array&#x27; ].&lt;string&gt;</code>** - _Optional_. Declaration of accepted VPAID vendors (whitelist).
Each specified String has to represent an unique identifier that can be found within the creative media file URL, most likely the domain name.
If omitted, the plugin tries to deliver VPAID creatives from each vendor.  

**deviceType : <code>string</code>** - _Optional_. Can be either `mobile`, `tablet`, `desktop` or `tv`. Defaulting to `null`.

**deviceScreenSize : <code>Object</code>** - _Optional_. Maximum size we can achieve when entering fullscreen.
- **width : <code>number</code>** - Screen width in pixel.
- **height : <code>number</code>** - Screen height in pixel.

**networkReachability : <code>string</code>** - _Optional_. Can be either `wwan`, `wifi` or `unreachable`. Defaulting to `null`.

**prefetchBuffetAds : <code>boolean</code>** - _Optional_. In order to replace invalid ads in a pod of ads instantly at runtime,
buffet ads from wrapper responses must be loaded up-front. `prefetchBuffetAds` determines whether stand-alone ads (buffet)
will be loaded together with the pod of ads or not. Defaulting to `false` if omitted.

**timeouts : <code>Object</code>** - _Optional_. Definition of `timeouts` that replace global settings for the upcoming ad-break.
Take a look at section [Global Config timeouts](#timeouts) for relevant information.

Example with `desiredBitrate`. Other variables will be ignored.
```javascript
adPlugin.initAdSlot(
	{
		// adBreak
	},{
 		desiredBitrate: 1000,
 		deviceScreenSize: {
 			width: 1024,
 			height: 768
 		}
 	}
 );
```

Example with `deviceType` and `networkReachability`, utilized to calculate an appropriat bitrate.
```javascript
adPlugin.initAdSlot(
	{
		// adBreak
	},{
		deviceType: 'tablet',
		deviceScreenSize: {
			width: 1024,
			height: 768
		},
		networkReachability: 'wifi'
 	}
 );
```
